package com.gic.haoban.manage.web.utils;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 生成token
 *
 * @author mozhu
 * @date 2021-12-10 14:53:34
 */
public class JwtUtil {

    private static final Logger logger = LoggerFactory.getLogger(JwtUtil.class);


    /**
     * 密钥 base64加密过的
     */
    public static final String SECRET_KEY = "Wb9^vZBjaYiFyhj@dDv!2FXSeEWxJauW";
    /**
     * 发行人
     */
    public static final String ISSUER = "gic_haoban";

    /**
     * 过期时间-7天
     */
    private static final long expire = 604800;


    /**
     * 生成token
     *
     * @param claims
     * @return
     */
    public static String genToken(Map<String, String> claims) {
        long time = System.currentTimeMillis();
        claims.put("requestTime", String.valueOf(time));
        //使用HMAC256进行加密
        Algorithm algorithm = Algorithm.HMAC256(SECRET_KEY);
        //创建jwt
        JWTCreator.Builder builder = JWT.create()
                .withIssuer(ISSUER)
                .withIssuedAt(new Date())
                .withExpiresAt(new Date(time + expire));
        //传入参数
        claims.forEach(builder::withClaim);
        //签名加密
        return builder.sign(algorithm);
    }

    /**
     * 解密token
     *
     * @param token
     * @return
     * @throws RuntimeException
     */
    public static Map<String, String> parseToken(String token) {
        Algorithm algorithm = null;
        try {
            //使用HMAC256进行加密
            algorithm = Algorithm.HMAC256(SECRET_KEY);
        } catch (IllegalArgumentException e) {
            logger.error("解密jwt失败:{}", e.getMessage(), e);
            return null;
        }
        //解密
        JWTVerifier verifier = JWT.require(algorithm).withIssuer(ISSUER).build();
        DecodedJWT jwt = verifier.verify(token);
        Map<String, Claim> map = jwt.getClaims();
        Map<String, String> resultMap = new HashMap<>();
        map.forEach((k, v) -> resultMap.put(k, v.asString()));
        return resultMap;
    }

    public static void main(String[] args) {
        String token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJwaG9uZU51bWJlciI6IjEzNTc2MjIzNDQ3IiwiaXNzIjoiZ2ljX2hhb2JhbiIsInd4RW50ZXJwcmlzZUlkIjoiY2E2NmEwMWI3OTQ3NGM0MGIzZTdjN2Y5M2RhZjFhM2IiLCJzdGFmZklkIjoiZWM0ODI0NzBiZmIxNGFlZjkxMTBhYTIwM2Y5MmRmYjgifQ.rcP4U42mvvNKiiJXWoHVL8ad7doFmfgt5LuigrI_m5I";
        Map<String, String> stringStringMap = parseToken(token);
        String s = genToken(stringStringMap);
        System.out.println(s);
    }
}
